package eu.eudat.documents.xml;

import eu.eudat.models.user.components.datasetprofile.Field;
import eu.eudat.models.user.components.datasetprofile.FieldSet;
import eu.eudat.models.user.components.datasetprofile.Section;
import eu.eudat.models.user.composite.DatasetProfilePage;
import eu.eudat.models.user.composite.PagedDatasetProfile;
import eu.eudat.services.forms.VisibilityRuleService;
import eu.eudat.utilities.builders.XmlBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import java.util.UUID;

/**
 * Created by ikalyvas on 3/5/2018.
 */
public class ExportXmlBuilder {

    public File build(PagedDatasetProfile pagedDatasetProfile, VisibilityRuleService visibilityRuleService) throws IOException {

        File xmlFile = new File(UUID.randomUUID() + ".xml");
        BufferedWriter writer = new BufferedWriter(new FileWriter(xmlFile, true));
        Document xmlDoc = XmlBuilder.getDocument();
        Element root = createPages(pagedDatasetProfile.getPages(), visibilityRuleService, xmlDoc);
        xmlDoc.appendChild(root);
        String xml = XmlBuilder.generateXml(xmlDoc);
        writer.write(xml);
        writer.close();
        return xmlFile;
    }

    public Element createPages(List<DatasetProfilePage> datasetProfilePages, VisibilityRuleService visibilityRuleService, Document element) {
        Element root = element.createElement("root");
        Element pages = element.createElement("pages");
        datasetProfilePages.forEach(item -> {
            Element page = element.createElement("page");
            page.appendChild(createSections(item.getSections(), visibilityRuleService, element));
            pages.appendChild(page);
        });
        root.appendChild(pages);
        return root;
    }

    public Element createSections(List<Section> sections, VisibilityRuleService visibilityRuleService, Document element) {
        Element elementSections = element.createElement("sections");
        sections.forEach(section -> {
            Element elementSection = element.createElement("section");
            if (visibilityRuleService.isElementVisible(section.getId())) {
                Element elementInnerSections = element.createElement("sections");
                Element compositeFields = element.createElement("composite-field");
                elementInnerSections.appendChild(createSections(section.getSections(), visibilityRuleService, element));
                compositeFields.appendChild(createCompositeFields(section.getCompositeFields(), visibilityRuleService, element));
                elementSection.appendChild(elementInnerSections);
                elementSections.appendChild(elementSection);
                elementSections.appendChild(compositeFields);
            }
        });
        return elementSections;
    }

    public Element createCompositeFields(List<FieldSet> compositeFields, VisibilityRuleService visibilityRuleService, Document element) {
        Element elementComposites = element.createElement("composite-fields");
        compositeFields.forEach(compositeField -> {
            if (visibilityRuleService.isElementVisible(compositeField.getId())) {
                Element composite = element.createElement("composite-field");
                composite.setAttribute("id", compositeField.getId());
                if (compositeField.getTitle() != null && !compositeField.getTitle().isEmpty()) {
                    Element title = element.createElement("title");
                    title.setTextContent(compositeField.getTitle());
                    composite.appendChild(title);
                }
                if (compositeField.getDescription() != null && !compositeField.getDescription().isEmpty()) {
                    Element title = element.createElement("description");
                    title.setTextContent(compositeField.getDescription());
                    composite.appendChild(title);
                }
                composite.appendChild(createFields(compositeField.getFields(), visibilityRuleService, element));
                elementComposites.appendChild(composite);

            }
        });
        return elementComposites;
    }

    public Element createFields(List<Field> fields, VisibilityRuleService visibilityRuleService, Document element) {
        Element elementFields = element.createElement("fields");
        fields.forEach(field -> {
            if (visibilityRuleService.isElementVisible(field.getId())) {
                Element elementField = element.createElement("field");
                elementField.setAttribute("id", field.getId());
                if (field.getValue() != null && !field.getValue().isEmpty()) {
                    elementField.setTextContent(field.getValue());
                }
                elementFields.appendChild(elementField);
            }
        });
        return elementFields;
    }
}
