package eu.eudat.models.project;

import eu.eudat.data.entities.Project;
import eu.eudat.data.entities.UserInfo;
import eu.eudat.models.DataModel;
import eu.eudat.models.files.ContentFile;
import eu.eudat.models.urls.DataManagementPlanUrlListing;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;


public class ProjectListingModel implements DataModel<eu.eudat.data.entities.Project, ProjectListingModel> {

    private UUID id;

    private String label;

    private String abbreviation;

    private String reference;

    private String uri;

    private String definition;

    private String startDate;

    private String endDate;

    private Project.Status status;

    private UserInfo creationUser;

    private String created;

    private String modified;

    private String description;

    private List<ContentFile> files;

    private List<DataManagementPlanUrlListing> dmps;


    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }

    public List<DataManagementPlanUrlListing> getDmps() {
        return dmps;
    }

    public void setDmps(List<DataManagementPlanUrlListing> dmps) {
        this.dmps = dmps;
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public String getAbbreviation() {
        return abbreviation;
    }

    public void setAbbreviation(String abbreviation) {
        this.abbreviation = abbreviation;
    }

    public String getReference() {
        return reference;
    }

    public void setReference(String reference) {
        this.reference = reference;
    }

    public String getUri() {
        return uri;
    }

    public void setUri(String uri) {
        this.uri = uri;
    }

    public String getDefinition() {
        return definition;
    }

    public void setDefinition(String definition) {
        this.definition = definition;
    }

    public short getStatus() {
        return status.getValue();
    }

    public void setStatus(Short status) {
        this.status = Project.Status.fromInteger(status);
    }

    public UserInfo getCreationUser() {
        return creationUser;
    }

    public void setCreationUser(UserInfo creationUser) {
        this.creationUser = creationUser;
    }

    public String getStartDate() {
        return startDate;
    }

    public void setStartDate(String startDate) {
        this.startDate = startDate;
    }

    public String getEndDate() {
        return endDate;
    }

    public void setEndDate(String endDate) {
        this.endDate = endDate;
    }

    public String getCreated() {
        return created;
    }

    public void setCreated(String created) {
        this.created = created;
    }

    public String getModified() {
        return modified;
    }

    public void setModified(String modified) {
        this.modified = modified;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public List<ContentFile> getFiles() {
        return files;
    }

    public void setFiles(List<ContentFile> files) {
        this.files = files;
    }

    @Override
    public ProjectListingModel fromDataModel(eu.eudat.data.entities.Project entity) {
        this.id = entity.getId();
        this.label = entity.getLabel();
        this.abbreviation = entity.getAbbreviation();
        this.reference = entity.getReference();
        this.uri = entity.getUri();
        this.definition = entity.getDefinition();
        this.startDate = entity.getStartdate() != null ? entity.getStartdate().toString() : null;
        this.endDate = entity.getEnddate() != null ? entity.getEnddate().toString() : null;
        this.setStatus(entity.getStatus());
        this.created = entity.getCreated().toString();
        this.modified = entity.getModified().toString();
        this.description = entity.getDescription();
        this.dmps = entity.getDmps().stream().map(item -> new DataManagementPlanUrlListing().fromDataModel(item)).collect(Collectors.toList());
        this.files = entity.getContent() != null ? Arrays.asList(new ContentFile(entity.getContent().getLabel(), UUID.fromString(entity.getContent().getUri().split(":")[1]), "final", entity.getContent().getExtension())) : Arrays.asList(new ContentFile("default.png", null, null, null));
        return this;
    }

    @Override
    public eu.eudat.data.entities.Project toDataModel() throws Exception {
        throw new Exception("Not Implemented");
    }

    @Override
    public String getHint() {
        return "projectListingItem";
    }
}
