import { DataTableRequest } from '../models/data-table/DataTableRequest';
import { DatasetListingModel } from '../models/datasets/DatasetListingModel';
import { DatasetCriteria } from '../models/criteria/dataset/DatasetCriteria';
import { Component, ViewChild, OnInit, AfterViewInit } from "@angular/core";
import { MatPaginator, MatSort, MatSnackBar } from "@angular/material";
import { Router, Params, ActivatedRoute } from "@angular/router";
import { TranslateService } from "@ngx-translate/core";
import { DataSource } from "@angular/cdk/table";

import { Observable } from "rxjs/Observable";
import { PageEvent } from '@angular/material';
import { DataManagementPlanService } from "@app/services/data-management-plan/data-management-plan.service";
import { DataManagementPlanModel } from "@app/models/data-managemnt-plans/DataManagementPlanModel";
import { DatasetCriteriaComponent } from '@app/shared/components/criteria/datasets/datasets-criteria.component';
import { DatasetProfileAdmin } from '@app/services/datasetProfileAdmin/datasetProfileAfmin.service';


@Component({
  selector: 'app-dataset-admin-listing-component',
  templateUrl: 'dataset-admin-listing.component.html',
  styleUrls: ['./dataset-admin-listing.component.scss'],
  providers: [DatasetProfileAdmin, DataManagementPlanService]
})
export class DatasetAdminListingComponent implements OnInit {

  @ViewChild(MatPaginator) _paginator: MatPaginator;
  @ViewChild(MatSort) sort: MatSort;
  @ViewChild(DatasetCriteriaComponent) criteria: DatasetCriteriaComponent;


  dataSource: DatasetDataSource | null;
  displayedColumns: String[] = ['label', 'dmp', 'profile', 'dataRepositories', 'registries', 'services', 'description', 'created', 'actions'];
  pageEvent: PageEvent;
  titlePrefix: String;
  dmpId: String;

  statuses = [
    { value: '0', viewValue: 'Active' },
    { value: '1', viewValue: 'Inactive' }
  ];

  constructor(
    private datasetService: DatasetProfileAdmin,
    private router: Router,
    private languageService: TranslateService,
    public snackBar: MatSnackBar,
    public route: ActivatedRoute,
    public dataManagementPlanService: DataManagementPlanService
  ) {

  }


  ngOnInit() {
    this.route.params.subscribe((params: Params) => {
      this.dmpId = params['dmpId'];
      if (this.dmpId != null) this.setDmpTitle(this.dmpId);
      this.criteria.setCriteria(this.getDefaultCriteria(this.dmpId));
      this.refresh();
      this.criteria.setRefreshCallback(() => this.refresh());
    });
  }

  setDmpTitle(dmpId: String) {
    this.dataManagementPlanService.getSingle(dmpId).map(data => data as DataManagementPlanModel)
      .subscribe(data => {
        this.titlePrefix = data.label;
      });
  }

  refresh() {
    this.dataSource = new DatasetDataSource(this.datasetService, this._paginator, this.sort, this.languageService, this.snackBar, this.criteria);
  }

  rowClick(rowId: String) {
    this.router.navigate(['form/' + rowId]);
  }

  getDefaultCriteria(dmpId: String): DatasetCriteria {
    const defaultCriteria = new DatasetCriteria();
    if (dmpId != null) {
      defaultCriteria.dmpIds.push(dmpId);
    }
    return defaultCriteria;
  }

  // makeItPublic(id: String) {
  //   debugger;
  //   this.datasetService.makeDatasetPublic(id).subscribe();
  // }

}

export class DatasetDataSource extends DataSource<DatasetListingModel> {

  totalCount = 0;
  isLoadingResults = false;

  constructor(
    private _service: DatasetProfileAdmin,
    private _paginator: MatPaginator,
    private _sort: MatSort,
    private _languageService: TranslateService,
    private _snackBar: MatSnackBar,
    private _criteria: DatasetCriteriaComponent
  ) {
    super();

  }

  connect(): Observable<DatasetListingModel[]> {
    const displayDataChanges = [
      this._paginator.page
      //this._sort.matSortChange
    ];


    return Observable.merge(...displayDataChanges)
      .startWith(null)
      .switchMap(() => {
        setTimeout(() => {
          this.isLoadingResults = true;
        });
        const startIndex = this._paginator.pageIndex * this._paginator.pageSize;
        let fields: Array<string> = new Array()
        if (this._sort.active) fields = this._sort.direction === "asc" ? ["+" + this._sort.active] : ["-" + this._sort.active];
        const request = new DataTableRequest<DatasetCriteria>(startIndex, this._paginator.pageSize, { fields: fields });
        request.criteria = this._criteria.criteria;
        return this._service.getPaged(request);
      })
      /*.catch((error: any) => {
        this._snackBar.openFromComponent(SnackBarNotificationComponent, {
          data: { message: 'GENERAL.SNACK-BAR.FORMS-BAD-REQUEST', language: this._languageService },
          duration: 3000,
          extraClasses: ['snackbar-warning']
        });
        //this._criteria.criteria.onCallbackError(error);
        return Observable.of(null);
      })*/
      .map(result => {
        setTimeout(() => {
          this.isLoadingResults = false;
        });
        return result;
      })
      .map(result => {
        if (!result) { return []; }
        if (this._paginator.pageIndex === 0) { this.totalCount = result.totalCount; }
        return result.data;
      });
  }

  disconnect() {
    // No-op
  }
}