import { RouterModule } from '@angular/router';
import { TranslateHttpLoader } from '@ngx-translate/http-loader';
import { DatasetService } from '../services/dataset/dataset.service';
import { DynamicFormModule } from '../form/dynamic-form.module';
import { TranslateLoader, TranslateModule, TranslateService } from '@ngx-translate/core';
import { DatasetWizardComponent } from './dataset-wizard/dataset-wizard.component';
import { DatasetRoutes } from './dataset.routes';
import { DatasetListingComponent } from './listing/dataset-listing.component';
import { DatasetEditorComponent } from './editor/dataset-editor.component';
import { CommonModule } from '@angular/common';
import { HttpClient, HttpClientModule } from '@angular/common/http';
import { NgModule } from '@angular/core';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';

import { SharedModule } from '../shared/shared.module'
@NgModule({
    imports: [
        CommonModule,
        FormsModule,
        HttpClientModule,
        SharedModule,
        RouterModule.forChild(DatasetRoutes),
        ReactiveFormsModule,
        DynamicFormModule,
        TranslateModule.forRoot({
            loader: {
                provide: TranslateLoader,
                useFactory: HttpLoaderFactory,
                deps: [HttpClient]
            }
        }),
    ],

    declarations: [
        DatasetListingComponent,
        DatasetEditorComponent,
        DatasetWizardComponent
    ],

    exports: [
        DatasetListingComponent,
        DatasetEditorComponent,
        DatasetWizardComponent,
        RouterModule
    ],
    providers: [
        DatasetService
    ]
})

export class DatasetModule {
    constructor(private translate: TranslateService) {
        translate.setDefaultLang('en');
        translate.use('en');
    }
}

export function HttpLoaderFactory(httpClient: HttpClient) {
    return new TranslateHttpLoader(httpClient, 'assets/lang/', '.json');
}