import { DatasetProfileModel } from '../../../models/datasets/DatasetProfileModel';
import { DatasetProfileCriteria } from '../../../models/criteria/dataset/DatasetProfileCriteria';
import { BaseErrorModel } from '../../../models/error/BaseErrorModel';
import { SnackBarNotificationComponent } from '../../../shared/components/notificaiton/snack-bar-notification.component';
import { JsonSerializer } from '../../../utilities/JsonSerializer';
import { RequestItem } from '../../../models/criteria/RequestItem';
import { ProjectCriteria } from '../../../models/criteria/project/ProjectCriteria';
import { DmpUsersModel } from '../../../models/dmpUsers/DmpUsersModel';
import { AutoCompleteConfiguration } from '../../../shared/components/autocomplete/AutoCompleteConfiguration';
import { ExternalSourcesItemModel } from '../../../models/external-sources/ExternalSourcesItemModel';
import { DataManagementPlanModel } from '../../../models/data-managemnt-plans/DataManagementPlanModel';
import { ProjectService } from '../../../services/project/project.service';
import { ExternalSourcesService } from '../../../services/external-sources/external-sources.service';
import { DataManagementPlanService } from '../../../services/data-management-plan/data-management-plan.service';
import { Component, ViewChild, OnInit, AfterViewInit, ViewEncapsulation, Input } from "@angular/core";
import { MatPaginator, MatSort, MatSnackBar } from "@angular/material";
import { Router, ActivatedRoute, Params } from "@angular/router";
import { TranslateService } from "@ngx-translate/core";
import { DataSource } from "@angular/cdk/table";
import { Observable } from "rxjs/Observable";
import { FormGroup } from '@angular/forms';
import { AutoCompleteChipConfiguration } from "@app/shared/components/autocompleteChips/AutoCompleteChipConfiguration";
import { BaseCriteria } from '@app/models/criteria/BaseCriteria';


@Component({
    selector: 'app-dmp-wizard-editor-component',
    templateUrl: 'dmp-wizard-editor.component.html',
    styleUrls: ['./dmp-wizard-editor.component.scss'],
    providers: [DataManagementPlanService, ExternalSourcesService, ProjectService],
    encapsulation: ViewEncapsulation.None
})
export class DataManagementPlanWizardEditorComponent implements AfterViewInit {


    isNew = true;
    @Input() formGroup: FormGroup = null;

    filteringOrganisationsAsync: boolean = false;
    filteringResearchersAsync: boolean = false;
    filteredProfilesAsync: boolean = false;
    filteredOrganisations: ExternalSourcesItemModel[];
    filteredResearchers: ExternalSourcesItemModel[];
    filteredProfiles: DatasetProfileModel[];

    projectAutoCompleteConfiguration: AutoCompleteConfiguration;
    organisationsAutoCompleteConfiguration: AutoCompleteChipConfiguration;
    createNewVersion;
    associatedUsers: Array<DmpUsersModel>
    labelDisabled: boolean = false;

    constructor(
        private dataManagementPlanService: DataManagementPlanService,
        private projectService: ProjectService,
        private externalSourcesService: ExternalSourcesService,
        private route: ActivatedRoute,
        public snackBar: MatSnackBar,
        public router: Router,
        public language: TranslateService,
        private _service: DataManagementPlanService
    ) {

    }

    ngAfterViewInit() {

        let projectRequestItem: RequestItem<ProjectCriteria> = new RequestItem();
        projectRequestItem.criteria = new ProjectCriteria();
        this.projectAutoCompleteConfiguration = new AutoCompleteConfiguration(this.projectService.getWithExternal.bind(this.projectService), projectRequestItem);

        let organisationRequestItem: RequestItem<BaseCriteria> = new RequestItem();
        organisationRequestItem.criteria = new BaseCriteria();
        this.organisationsAutoCompleteConfiguration = new AutoCompleteChipConfiguration(this.externalSourcesService.searchDMPOrganizations.bind(this.externalSourcesService), organisationRequestItem);

        this.route.data.subscribe(value => {
            if (value.clone == false && this.formGroup.get("label").value) {
                this.labelDisabled = true;
            }
            this.formGroup.controls["version"].disable();
        })
    }

    formSubmit(): void {
        //this.touchAllFormFields(this.formGroup);
        if (!this.isFormValid()) { return; }
        this.onSubmit();
    }

    public isFormValid() {
        return this.formGroup.valid;
    }

    onSubmit(): void {
        this.dataManagementPlanService.createDataManagementPlan(this.formGroup.getRawValue()).subscribe(
            complete => this.onCallbackSuccess(),
            error => this.onCallbackError(error)
        );
    }

    onCallbackSuccess(): void {
        this.snackBar.openFromComponent(SnackBarNotificationComponent, {
            data: { message: this.isNew ? 'GENERAL.SNACK-BAR.SUCCESSFUL-CREATION' : 'GENERAL.SNACK-BAR.SUCCESSFUL-UPDATE', language: this.language },
            duration: 3000,
            extraClasses: ['snackbar-success']
        })
        this.router.navigate(['/dmps']);
    }

    onCallbackError(error: any) {
        this.setErrorModel(error.error);
        //this.validateAllFormFields(this.formGroup);
    }

    public setErrorModel(errorModel: BaseErrorModel) {
        Object.keys(errorModel).forEach(item => {
            // (<any>this.dataManagementPlan.errorModel)[item] = (<any>errorModel)[item];
        })
    }

    public cancel(): void {
        this.router.navigate(['/dmps']);
    }

    filterOrganisations(value: string): void {

        this.filteredOrganisations = undefined;
        if (value) {
            this.filteringOrganisationsAsync = true;

            this.externalSourcesService.searchDMPOrganizations(value).subscribe(items => {
                this.filteredOrganisations = items;
                this.filteringOrganisationsAsync = false;

                // this.filteredOrganisations = items.filter((filteredObj: any) => {
                //   return this.objectsModel ? this.objectsModel.indexOf(filteredObj) < 0 : true;
                // });

            });

        }
    }

    filterResearchers(value: string): void {

        this.filteredResearchers = undefined;
        if (value) {
            this.filteringResearchersAsync = true;

            this.externalSourcesService.searchDMPResearchers({ criteria: { name: value, like: null } }).subscribe(items => {
                this.filteredResearchers = items;
                this.filteringResearchersAsync = false;

                // this.filteredOrganisations = items.filter((filteredObj: any) => {
                //   return this.objectsModel ? this.objectsModel.indexOf(filteredObj) < 0 : true;
                // });

            });
        }
    }

    filterProfiles(value: string): void {

        this.filteredProfiles = undefined;
        if (value) {
            this.filteredProfilesAsync = true;

            // this.externalSourcesService.searchDMPProfiles(value).subscribe(items => {
            //   this.filteredProfiles = items;
            //   this.filteredProfilesAsync = false;

            //   // this.filteredOrganisations = items.filter((filteredObj: any) => {
            //   //   return this.objectsModel ? this.objectsModel.indexOf(filteredObj) < 0 : true;
            //   // });

            // });

            const request = new RequestItem<DatasetProfileCriteria>();
            let criteria = new DatasetProfileCriteria();
            criteria.like = value;
            request.criteria = criteria;
            this._service.searchDMPProfiles(request).subscribe(items => {
                this.filteredProfiles = items;
                this.filteredProfilesAsync = false;
            });
        }
    }

}