import { JsonSerializer } from '../utilities/JsonSerializer';
import { UserInvitationCriteria } from '../models/criteria/invitation/UserInvitationCriteria';
import { RequestItem } from '../models/criteria/RequestItem';
import { Invitation } from '../models/invitation/Invitation';
import { InvitationService } from '../services/invitation/invitation.service';
import { User } from '../models/invitation/User';
import { Component, OnInit, Inject } from "@angular/core";
import { FormGroup } from '@angular/forms';
import { Params, ActivatedRoute, Router } from '@angular/router';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material';

@Component({
    selector: 'app-invitation-component',
    templateUrl: 'invitation.component.html',
    providers: [InvitationService]

})
export class InvitationComponent implements OnInit {

    private formGroup: FormGroup;

    private filteredUsersAsync: boolean = false;

    private filteredUsers: User[];

    constructor(
        private invitationService: InvitationService,
        private route: ActivatedRoute,
        public router: Router,
        public dialogRef: MatDialogRef<InvitationComponent>,
        @Inject(MAT_DIALOG_DATA) public data: any
    ) { }

    ngOnInit(): void {
        let invitation = new Invitation();
        invitation.dataManagementPlan = this.data.dmpId;
        this.formGroup = invitation.buildForm();
    }


    send(value: any) {
        this.invitationService.inviteUsers(this.formGroup.value).subscribe(
            null, null, () => this.dialogRef.close()
        );
    }

    filterUsers(value: string): void {
        this.filteredUsers = undefined;
        if (value) {
            this.filteredUsersAsync = true;
            let request = new RequestItem<UserInvitationCriteria>();
            request.criteria = { like: value }
            this.invitationService.getUsers(request).subscribe(items => {
                this.filteredUsers = JsonSerializer.fromJSONArray(items, User);
                if (!this.filteredUsers || this.filteredUsers.length == 0) {
                    let user = new User();
                    user.email = value;
                    user.name = value;
                    this.filteredUsers.push(user)
                }
                this.filteredUsersAsync = false;
            });
        }
    }
}