import { DatasetProfileDefinitionModel } from '../DatasetProfileDefinitionModel';
import { DatasetProfileModel } from '../datasetprofile/DatasetProfileModel';
import { DataManagementPlanModel } from '../data-managemnt-plans/DataManagementPlanModel';
import { Serializable } from "../Serializable";
import { ValidationContext } from "../../utilities/validators/ValidationContext";
import { FormGroup, FormBuilder, FormControl, Validators } from "@angular/forms";
import { BackendErrorValidator } from "../../utilities/validators/BackendErrorValidator";
import { BaseErrorModel } from "../error/BaseErrorModel";
import { ExternalSourcesItemModel } from "../external-sources/ExternalSourcesItemModel";
import { ServiceModel } from "../services/ServiceModel";
import { JsonSerializer } from "../../utilities/JsonSerializer";
import { RegisterModel } from "../registers/RegisterModel";
import { DataRepositoryModel } from "../dataRepositories/DataRepositoryModel";
import { ExternalDatasetModel } from '@app/models/external-dataset/ExternalDatasetModel';

export class DatasetWizardModel implements Serializable<DatasetWizardModel> {
	public id: String;
	public label: String;
	public profile: DatasetProfileModel = new DatasetProfileModel();
	public uri: String;
	public status: String;
	public description: String;
	public services: ServiceModel[] = [];
	public registries: RegisterModel[] = [];
	public dataRepositories: DataRepositoryModel[] = [];
	public externalDatasets: ExternalDatasetModel[] = [];
	public dmp: DataManagementPlanModel = new DataManagementPlanModel();
	public datasetProfileDefinition;
	public errorModel: BaseErrorModel = new BaseErrorModel();

	fromJSONObject(item: any): DatasetWizardModel {
		this.id = item.id;
		this.label = item.label;
		this.profile = item.profile;
		this.uri = item.uri;
		this.status = item.status;
		this.description = item.description;
		this.services = JsonSerializer.fromJSONArray(item.services, ServiceModel);
		this.registries = JsonSerializer.fromJSONArray(item.registries, RegisterModel);
		this.dataRepositories = JsonSerializer.fromJSONArray(item.dataRepositories, DataRepositoryModel);
		this.dmp = JsonSerializer.fromJSONObject(item.dmp, DataManagementPlanModel);
		this.profile = JsonSerializer.fromJSONObject(item.profile, DatasetProfileModel);
		this.datasetProfileDefinition = JsonSerializer.fromJSONObject(item.datasetProfileDefinition, DatasetProfileDefinitionModel);
		return this;
	}

	buildForm(context: ValidationContext = null, disabled: boolean = false): FormGroup {
		if (context == null) { context = this.createValidationContext(); }
		const formGroup = new FormBuilder().group({
			id: [{ value: this.id, disabled: disabled }, context.getValidation('id').validators],
			label: [{ value: this.label, disabled: disabled }, context.getValidation('label').validators],
			uri: [{ value: this.uri, disabled: disabled }, context.getValidation('uri').validators],
			status: [{ value: this.status, disabled: disabled }, context.getValidation('status').validators],
			description: [{ value: this.description, disabled: disabled }, context.getValidation('description').validators],
			services: [{ value: this.services, disabled: disabled }, context.getValidation('services').validators],
			registries: [{ value: this.registries, disabled: disabled }, context.getValidation('registries').validators],
			dataRepositories: [{ value: this.dataRepositories, disabled: disabled }, context.getValidation('dataRepositories').validators],
			externalDatasets: [{ value: this.externalDatasets, disabled: disabled }, context.getValidation('externalDatasets').validators],
			dmp: [{ value: this.dmp, disabled: disabled }, context.getValidation('dmp').validators]
		});
		if (this.datasetProfileDefinition) formGroup.addControl("datasetProfileDefinition", this.datasetProfileDefinition.buildForm())
		formGroup.addControl("profile", this.profile.buildForm())
		return formGroup;
	}

	createValidationContext(): ValidationContext {
		const baseContext: ValidationContext = new ValidationContext();
		baseContext.validation.push({ key: 'id', validators: [BackendErrorValidator(this.errorModel, 'id')] });
		baseContext.validation.push({ key: 'label', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'label')] });
		baseContext.validation.push({ key: 'profile', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'profile')] });
		baseContext.validation.push({ key: 'uri', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'uri')] });
		baseContext.validation.push({ key: 'status', validators: [BackendErrorValidator(this.errorModel, 'status')] });
		baseContext.validation.push({ key: 'description', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'description')] });
		baseContext.validation.push({ key: 'services', validators: [BackendErrorValidator(this.errorModel, 'services')] });
		baseContext.validation.push({ key: 'registries', validators: [BackendErrorValidator(this.errorModel, 'registries')] });
		baseContext.validation.push({ key: 'dataRepositories', validators: [BackendErrorValidator(this.errorModel, 'dataRepositories')] });
		baseContext.validation.push({ key: 'externalDatasets', validators: [BackendErrorValidator(this.errorModel, 'externalDatasets')] });
		baseContext.validation.push({ key: 'dmp', validators: [BackendErrorValidator(this.errorModel, 'dmp')] });   //TODO
		baseContext.validation.push({ key: 'datasetProfileDefinition', validators: [BackendErrorValidator(this.errorModel, 'datasetProfileDefinition')] });   //TODO

		return baseContext;
	}
}