import { Component } from '@angular/core';
import { Principal } from '../../../models/login/Principal';
import { AuthService } from '../../../services/auth/auth.service';
import { LanguageResolverService } from '../../../services/language-resolver/language-resolver.service';

@Component({
	selector: 'app-navigation',
	templateUrl: 'navigation.component.html',
	styleUrls: ['./navigation.component.css'],
	providers: []
})

export class NavigationComponent {
	invert = false;

	constructor(private authentication: AuthService, private languageResolver: LanguageResolverService) {

	}

	public logout(): void {
		this.authentication.logout();
	}

	public isAuthenticated(): boolean {
		return !(!this.authentication.current())
	}

	public isAdmin(): boolean {
		if (!this.authentication.current()) { return false }
		const principalRoles = this.authentication.current().appRoles;
		for (let i = 0; i < principalRoles.length; i++) {
			if (principalRoles[i] === Principal.AppRole.Admin) {
				return true;
			}
		}
		return false;
	}

	public getPrincipalName(): string {
		const principal: Principal = this.authentication.current();
		if (principal) { return principal.name; }
		return '';
	}
}
